<?php
/*
 * Parser BBcode inspirer fortement de celui de FluxBB 1.3 rev641
 *
 * @copyright Copyright (C) 2008 FluxBB.org, based on code copyright (C) 2002-2008 PunBB.org
 * @license http://www.gnu.org/licenses/gpl.html GPL version 2 or higher
 * @package FluxBB
 */

function split_text4BBcode($text, $start, $end, &$errors, $retab = true)
{
    global $forum_config, $lang_common;

    $tokens = explode($start, $text);

    $outside[] = $tokens[0];

    $num_tokens = count($tokens);
    for ($i = 1; $i < $num_tokens; ++$i)
    {
        $temp = explode($end, $tokens[$i]);

        if (count($temp) != 2)
        {
            $errors[] = $lang_common['BBCode code problem'];
            return array(null, array($text));
        }
        $inside[] = $temp[0];
        $outside[] = $temp[1];
    }

    if ($forum_config['o_indent_num_spaces'] != 8 && $retab)
    {
        $spaces = str_repeat(' ', $forum_config['o_indent_num_spaces']);
        $inside = str_replace("\t", $spaces, $inside);
    }

    return array($inside, $outside);
}

function praserBBcode($text) {
    // If the message contains a code tag we have to split it up (text within [code][/code] shouldn't be touched)
    static $brBBcode;
    if (!isset($brBBcode))
        $brBBcode = convstr::xml2motor(nl2br("\n"));

    $text = str_replace($brBBcode,"\n",$text);

    if (strpos($text, '[code]') !== false && strpos($text, '[/code]') !== false)
    {
        list($inside, $outside) = split_text4BBcode($text, '[code]', '[/code]', $errors);
        $text = implode("\0", $outside);
    }

    if (strpos($text, '[quote') !== false)
    {
        $text = preg_replace('#\[quote\]\s*#', '</p><div class="quotebox"><blockquote><p>', $text);
        $text = preg_replace('#\[quote\$egal\$(&quot;|"|\'|)(.*?)\\1\]#se', '"</p><div class=\"quotebox\"><cite>".str_replace(array(\'[\', \'\\"\'), array(\'&#91;\', \'"\'), \'$2\')." ".$lang_common[\'wrote\'].":</cite><blockquote><p>"', $text);
        $text = preg_replace('#\s*\[\/quote\]#S', '</p></blockquote></div><p>', $text);
    }

    $pattern[] = '#\[b\](.*?)\[/b\]#ms';
    $pattern[] = '#\[i\](.*?)\[/i\]#ms';
    $pattern[] = '#\[u\](.*?)\[/u\]#ms';
    $pattern[] = '#\[colou?r=([a-zA-Z]{3,20}|\#[0-9a-fA-F]{6}|\#[0-9a-fA-F]{3})](.*?)\[/colou?r\]#ms';
    $pattern[] = '#\n*\[h\](.*?)\[/h\]\n*#ms';
    $pattern[] = '#\[h\](.*?)\[/h\]#ms';

    $replace[] = '<strong>$1</strong>';
    $replace[] = '<em>$1</em>';
    $replace[] = '<span class="bbu">$1</span>';
    $replace[] = '<span style="color: $1">$2</span>';
    $replace[] = '</p><h4>$1</h4><p>';
    $replace[] = '</p><h4>$1</h4><p>';

    $pattern[] = '#\[img\]((ht|f)tps?://)([^\s<"]*?)\[/img\]#e';
    $pattern[] = '#\[img\$egal\$([^\[]*?)\]((ht|f)tps?://)([^\s<"]*?)\[/img\]#e';

    $replace[] = 'handle_img_tag(\'$1$3\', false)';
    $replace[] = 'handle_img_tag(\'$2$4\', false, \'$1\')';

    $pattern[] = '#\[url\]([^\[]*?)\[/url\]#';
    $pattern[] = '#\[url\$egal\$([^\[]+?)\](.*?)\[/url\]#';
    $pattern[] = '#\[email\]([^\[]*?)\[/email\]#';
    $pattern[] = '#\[email\$egal\$([^\[]+?)\](.*?)\[/email\]#';

    $replace[] = '<a href="$1">$1</a>';
    $replace[] = '<a href="$1">$2</a>';
    $replace[] = '<a href="mailto:$1">$1</a>';
    $replace[] = '<a href="mailto:$1">$2</a>';

    // This thing takes a while! :)
    $text = preg_replace($pattern, $replace, $text);


    // Deal with newlines, tabs and multiple spaces
    $pattern = array("\n", "\t", '  ', '  ');
    $replace = array('<br />', '&nbsp; &nbsp; ', '&nbsp; ', ' &nbsp;');
    $text = str_replace($pattern, $replace, $text);

    // If we split up the message before we have to concatenate it together again (code tags)
    if (isset($inside))
    {
        $outside = explode("\0", $text);
        $text = '';

        $num_tokens = count($outside);

        for ($i = 0; $i < $num_tokens; ++$i)
        {
            $text .= $outside[$i];
            if (isset($inside[$i]))
                $text .= '</p><div class="codebox"><pre><code>'.trim($inside[$i], "\n\r").'</code></pre></div><p>';
        }
    }

    return $text;
}
